<?php
namespace App\Http\Controllers\IoT;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use DB;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Contracts\Encryption\DecryptException;
use Validator;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Hash;
use App\Mail\DemoEmail;
use App\Http\Utility\Common;
use App\Http\Utility\Defaults;
use App\Http\Service\DataService;
use App\DeviceInfo;
use App\Device_battery;

class IotController extends Controller
{
	public function __construct(){
 		DB::enableQueryLog();
	}

	public function registerDevice(){
		$data  				=	new DataService();
		$params 			= 	json_decode(file_get_contents('php://input'), TRUE);
		$authorisation		= 	$params['R'];
		$device_id 			= 	$params['D'];
		$device_type		= 	$params['Y'];		//5=5 port device, 12= 12 port device
		$device_location	= 	$params['Z'];
		$locatArr 			= 	explode(',',$device_location);
		$device_lat 		= 	$locatArr[0];
		$device_long 		= 	$locatArr[1];
		$device_version 	= 	$params['V'];
		$iot_number 		= 	$params['C'];
		$device_module 		= 	$params['Q'];
		$address 			= 	$this->getaddress($device_lat,$device_long);
		$current_timestamp	=	Defaults::getCurrentTimeStamp();

		$insert = array(
		    "device_id" => $device_id,
		    "device_password" => 123456,
			"device_type" => $device_type,
			"device_lat" => $device_lat,
			"device_long" => $device_long,
			"device_version" => $device_version,
			"iot_number" => $iot_number,
			"device_module" => $device_module
		);
		if(!empty($address)){
			$insert["device_address"] = $address;
		}

		$isExist = DeviceInfo::where('device_id',$device_id)->count();

		if($isExist){
			DeviceInfo::where('device_id',$device_id)->update($insert);
		}else{
			$insert["device_id"] = $device_id;
			$device_obj = new DeviceInfo($insert);
			$device_obj->save();
		}
	
		$data    = array("G"=>$params['G'],"P"=>$_SERVER['REMOTE_ADDR'],
			"O"=>$_SERVER['SERVER_PORT'],"N"=>$current_timestamp,
			"S"=>"","X"=>"",
			"Z"=>"","A"=>"",
			"C"=>$device_type,"W"=>""
		);
		if(1){
			$data["M"] = "";
			$data["V"] = "";
			$data["U"] = "";
		}

		$responsedata = Defaults::encode($data);
 		return $responsedata;
	}

	public function hearBeat(){
		//After successful registration, the device will first send a special heartbeat packet to the communication server, and then send the heartbeat packet.
        
        /* Get information from Device to server */
        
		$data  					=	new DataService();
		$params 				= 	json_decode(file_get_contents('php://input'), TRUE);
		$indication_from		= 	$params['T'];	//T-from device , W-from server
		$heartbeat_type 		= 	$params['R'];	//T-Special Heartbeat/I-Heartbeat/Q-Forced Heartbeat
		$device_type			= 	$params['Y'];	//5=5 port device, 12= 12 port device
		$password				= 	$params['M'];   //Password
		$device_id 				= 	$params['D'];   //Device id
		$available_batery 	    = 	$params['J'];	//Quantity can be rented
		$to_be_return_batery    = 	$params['H'];   //Quantity can be returned
		$batery_list			= 	$params['L']; //Battery information list
		$current_timestamp	    =	Defaults::getCurrentTimeStamp();
		
		$device_exist = DeviceInfo::where(['device_id'=>$device_id])->count();
		if($device_exist){
		    /* Return information from server to device */
    		$data    = array("C"=>"O","G"=>$params['G'],
    			"R"=>"T","D"=>$params['D'],
    			"N"=>$current_timestamp,"A"=>$_SERVER['SERVER_PORT']
    		);
    	}else{
		    $device_authorise = DeviceInfo::where(['device_id'=>$device_id,'device_password'=>$password])->count();
		    if(!$device_authorise){
    		    /* Return information from server to device */
        		$data    = array("C"=>"N","G"=>$params['G'],
        			"R"=>"T","D"=>$params['D'],
        			"N"=>$current_timestamp,"A"=>$_SERVER['SERVER_PORT']
        		);
    		}else{
    		    
        		$device_status = 0;
        		$available_for_rental = $available_batery;
        		$empty_for_return = $to_be_return_batery;
        		
        		if($heartbeat_type === 'T'){
        			$device_status = 1;
        		}
        		DeviceInfo::where(['device_id'=>$device_id])->update(['device_status'=>$device_status,
        		    'available_for_rental'=>$available_for_rental,'empty_for_return'=>$empty_for_return]);
        
        		$insert = 0;
        		for($re = 0; $re < count($batery_list); $re++){
        			$batery_exist = Device_battery::where(['battery_id'=>$batery_list[$re]['B1']])->count();
        			if($batery_exist){
        				Device_battery::where(['battery_id'=>$batery_list[$re]['B1']])->update([
        					'device_id'=>$device_id,'battery_port'=>$batery_list[$re]['B2'],'battery_unit'=>$batery_list[$re]['B3'],
        					'battery_switch'=>$batery_list[$re]['B4'],'battery_logo_bit'=>$batery_list[$re]['B5']
        				]);
        			}else{
        				$insert = 1;
        				$arr[] = array(
        					'device_id'=>$device_id,'battery_id'=>$batery_list[$re]['B1'],
        					'battery_port'=>$batery_list[$re]['B2'],'battery_unit'=>$batery_list[$re]['B3'],
        					'battery_switch'=>$batery_list[$re]['B4'],'battery_logo_bit'=>$batery_list[$re]['B5']
        				);
        				
        			}
        		}
        
        		if($insert){
        			Device_battery::insert($arr);
        		}
        		
        		/* Return information from server to device */
        		$data    = array("C"=>"Y","G"=>$params['G'],
        			"R"=>"T","D"=>$params['D'],
        			"N"=>$current_timestamp,"A"=>$_SERVER['SERVER_PORT']
        		);
    		}
		}
		$responsedata = Defaults::encode($data);
	 	return $responsedata;
	}

    public function shortHearBeat(){
		//After successful registration, the device will first send a special heartbeat packet to the communication server, and then send the heartbeat packet.
        
        /* Get information from Device to server */
        
		$data  					=	new DataService();
		$params 				= 	json_decode(file_get_contents('php://input'), TRUE);
		$indication_from		= 	$params['T'];	//T-from device , W-from server
		$heartbeat_type 		= 	$params['R'];	//SI-ShortHeartbeat
		$device_type			= 	$params['Y'];	//5=5 port device, 12= 12 port device
		$password				= 	$params['M'];   //Password
		$device_id 				= 	$params['D'];   //Device id
		$available_batery 	    = 	$params['J'];	//Quantity can be rented
		$to_be_return_batery    = 	$params['H'];   //Quantity can be returned
		$batery_list			= 	$params['L']; //Battery information list
		$current_timestamp	    =	Defaults::getCurrentTimeStamp();
		
		$device_exist = DeviceInfo::where(['device_id'=>$device_id])->count();
		if(!$device_exist){
		    /* Return information from server to device */
    		$data    = array("C"=>"O","G"=>$params['G'],
    			"R"=>"T","D"=>$params['D'],
    			"N"=>$current_timestamp,"A"=>$_SERVER['SERVER_PORT']
    		);
    	}else{
		    $device_authorise = DeviceInfo::where(['device_id'=>$device_id,'device_password'=>$password])->count();
		    if(!$device_authorise){
    		    /* Return information from server to device */
        		$data    = array("C"=>"N","G"=>$params['G'],
        			"R"=>"T","D"=>$params['D'],
        			"N"=>$current_timestamp,"A"=>$_SERVER['SERVER_PORT']
        		);
    		}else{
    		    
        		$device_status = 0;
        		$available_for_rental = $available_batery;
        		$empty_for_return = $to_be_return_batery;
        		if($heartbeat_type === 'T'){
        			$device_status = 1;
        		}
        		DeviceInfo::where(['device_id'=>$device_id])->update(['device_status'=>$device_status,
        		    'available_for_rental'=>$available_for_rental,'empty_for_return'=>$empty_for_return]);
        
        		$insert = 0;
        		for($re = 0; $re < count($batery_list); $re++){
        			$batery_exist = Device_battery::where(['battery_id'=>$batery_list[$re]['B1']])->count();
        			if($batery_exist){
        				Device_battery::where(['battery_id'=>$batery_list[$re]['B1']])->update([
        					'device_id'=>$device_id,'battery_port'=>$batery_list[$re]['B2'],'battery_unit'=>$batery_list[$re]['B3'],
        					'battery_switch'=>$batery_list[$re]['B4'],'battery_logo_bit'=>$batery_list[$re]['B5']
        				]);
        			}else{
        				$insert = 1;
        				$arr[] = array(
        					'device_id'=>$device_id,'battery_id'=>$batery_list[$re]['B1'],
        					'battery_port'=>$batery_list[$re]['B2'],'battery_unit'=>$batery_list[$re]['B3'],
        					'battery_switch'=>$batery_list[$re]['B4'],'battery_logo_bit'=>$batery_list[$re]['B5']
        				);
        				
        			}
        		}
        
        		if($insert){
        			Device_battery::insert($arr);
        		}
        		
        		/* Return information from server to device */
        		$data    = array("C"=>"Y","G"=>$params['G'],
        			"R"=>"T","D"=>$params['D'],
        			"N"=>$current_timestamp,"A"=>$_SERVER['SERVER_PORT']
        		);
    		}
		}
		$responsedata = Defaults::encode($data);
	 	return $responsedata;
	}
    
    
	public function getaddress($lat,$lng){
		$url 			= 	'https://maps.googleapis.com/maps/api/geocode/json?key=AIzaSyAQtiGmdJ8a92f4o9DTml0yDPYIFvejybI&&latlng='.trim($lat).','.trim($lng).'&sensor=false';
		$json 		= 	@file_get_contents($url);
	  	$data			=	json_decode($json);
	  	if(!empty($data)){
	  		$status = $data->status;
	  		if($status=="OK")
	     	{
	     	  return $data->results[0]->formatted_address;
	     	}
	     	else
	     	{
	     	  return false;
	     	}
	  	}else
	  	{
	  	  return false;
	  	}
	}
}